// Description: FreeOTFE Hash Device Driver API
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#ifndef _FreeOTFEHashAPI_H
#define _FreeOTFEHashAPI_H   1

// FreeOTFEAPI.h is included for the definition of DEVICE_FREEOTFE_ROOT, etc
#include "FreeOTFEAPI.h"


// Required for definition of ULONG
#include <windef.h>




// Following is required for the "_T" macro
/// #include <tchar.h>

#include <initguid.h>  // Required for DEFINE_GUID and other GUID defintions

// #define _T(x) L ## x

// =========================================================================
// Const definitions

#define MAX_HASH_TITLE 256


// Maximum hash length
// This should be set to the MAXIMUM hash length that the any hash may generate.
// This const is used when generating sector IVs for volumes which use the hash
// of the sector ID as the sector IV. It is used then in order to allocate a suitably sized
// buffer before hashing the sector ID.
// Note: This value is in *bits*
#define FREEOTFE_MAX_HASH_LENGTH 1024



#define DEVICE_HASH_BASE_NAME          L"\\FreeOTFEHash"

#define DEVICE_HASH_DIR_NAME           DEVICE_FREEOTFE_ROOT L"\\Hash"
#define DEVICE_HASH_SYMLINK_DIR_NAME   DEVICE_SYMLINK_FREEOTFE_ROOT
#define DEVICE_HASH_SYMLINK_PREFIX     DEVICE_HASH_SYMLINK_DIR_NAME DEVICE_HASH_BASE_NAME



#define IOCTL_FREEOTFEHASH_IDENTIFYDRIVER    CTL_CODE(                     \
                                                FILE_DEVICE_UNKNOWN, \
                                                0x800,               \
                                                METHOD_BUFFERED,     \
                                                FILE_READ_DATA       \
                                               )

#define IOCTL_FREEOTFEHASH_IDENTIFYSUPPORTED    CTL_CODE(                     \
                                                FILE_DEVICE_UNKNOWN, \
                                                0x801,               \
                                                METHOD_BUFFERED,     \
                                                FILE_READ_DATA       \
                                               )

#define IOCTL_FREEOTFEHASH_HASHDATA    CTL_CODE(                     \
                                                FILE_DEVICE_UNKNOWN, \
                                                0x802,               \
                                                METHOD_BUFFERED,     \
                                                FILE_READ_DATA       \
                                               )

#define IOCTL_FREEOTFEHASH_INTLDETAILS  CTL_CODE(                     \
                                                FILE_DEVICE_UNKNOWN, \
                                                0x803,               \
                                                METHOD_BUFFERED,     \
                                                FILE_READ_DATA       \
                                               )


// =========================================================================
// Type definitions

typedef struct _HASH {
    GUID HashGUID;
    char Title[MAX_HASH_TITLE];
    ULONG VersionID;
    int Length;  // In bits
    int BlockSize;  // In bits
} HASH, *PHASH;


typedef struct _DIOC_HASH_IDENTIFYDRIVER {    
    GUID DriverGUID;
    char Title[MAX_HASH_TITLE];
    ULONG VersionID;
    unsigned int CountHashes;
} DIOC_HASH_IDENTIFYDRIVER, *PDIOC_HASH_IDENTIFYDRIVER;


typedef struct _DIOC_HASH_IDENTIFYSUPPORTED {    
    unsigned int BufCount;  // The number of *elements* in the "Hashes" array
    HASH Hashes[1];  // Variable length
} DIOC_HASH_IDENTIFYSUPPORTED, *PDIOC_HASH_IDENTIFYSUPPORTED;


typedef struct _DIOC_HASH_DATA_IN {
    GUID HashGUID;
    unsigned int DataLength;  // In bits
    unsigned char Data[1];  // Variable length
} DIOC_HASH_DATA_IN, *PDIOC_HASH_DATA_IN;


// IF THIS IS CHANGED - remember to change the definition of how 
// "userBufferSizeBytes" is calculated
typedef struct _DIOC_HASH_DATA_OUT {
    unsigned int HashLength;  // In bits
    unsigned char Hash[1];  // Variable length
} DIOC_HASH_DATA_OUT, *PDIOC_HASH_DATA_OUT;


// This definition must be kept in sync with "ImpHashHashData"
typedef NTSTATUS (* PDataHashFn)(
                                 GUID*,
                                 unsigned int,
                                 unsigned char*,
                                 unsigned int*, 
                                 unsigned char*
                                );

// This definition must be kept in sync with "GetHashDetails"
typedef NTSTATUS (* PHashDetailsFn)(
                                    GUID*,
                                    HASH*
                                   );

typedef struct _DIOC_HASH_INTLDETAILS {
    PHashDetailsFn FnHashDetails;
    PDataHashFn FnHash;    
} DIOC_HASH_INTLDETAILS, *PDIOC_HASH_INTLDETAILS;


// =========================================================================
// =========================================================================

#endif

