// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


#include "FreeOTFEHashDriver.h"
#include "FreeOTFEHashImpl.h"
#include "FreeOTFEHashNull.h"

#include "FreeOTFEDebug.h"
#include "FreeOTFElib.h"


// =========================================================================
// Hash driver init function
// devExt - The device extension to be initialized
NTSTATUS
ImpHashDriverExtDetailsInit(
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;
    int idx = -1;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsInit\n"));


    // -- POPULATE DRIVER IDENTIFICATION --
    RtlZeroMemory(devExt->DriverTitle, sizeof(devExt->DriverTitle));
    RtlCopyMemory(
                  devExt->DriverTitle,
                  DRIVER_TITLE,
                  strlen(DRIVER_TITLE)
                 );

    devExt->DriverGUID = DRIVER_GUID;
    devExt->DriverVersionID = DRIVER_HASH_VERSION;


    // -- POPULATE HASHES SUPPORTED --
    devExt->HashDetails = ExAllocatePool(
                                         NonPagedPool,
                                         (sizeof(HASH) * HASHES_SUPPORTED)
                                        );    

    devExt->HashCount = HASHES_SUPPORTED;



    // -- Null --
    idx++;
    RtlZeroMemory(devExt->HashDetails[idx].Title, MAX_HASH_TITLE);
    RtlCopyMemory(
                  devExt->HashDetails[idx].Title,
                  DRIVER_HASH_TITLE_NULL,
                  strlen(DRIVER_HASH_TITLE_NULL)
                 );

    devExt->HashDetails[idx].Length = -1;
    devExt->HashDetails[idx].BlockSize = -1;
    devExt->HashDetails[idx].VersionID = DRIVER_HASH_IMPL_VERSION;
    devExt->HashDetails[idx].HashGUID = HASH_GUID_NULL;



    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsInit\n"));

    return status;
}


// =========================================================================
// Hash driver cleardown function
// devExt - The device extension to be cleared down
NTSTATUS
ImpHashDriverExtDetailsCleardown(    
    IN OUT DEVICE_EXTENSION* devExt
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashDriverExtDetailsCleardown\n"));

    if (devExt->HashDetails != NULL)
        {
        ExFreePool(devExt->HashDetails);
        }

    devExt->HashDetails = NULL;
    devExt->HashCount = 0;

    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashDriverExtDetailsCleardown\n"));

    return status;
}


// =========================================================================
// Hash function
NTSTATUS
ImpHashHashData(
    IN      GUID* HashGUID,
    IN      unsigned int DataLength,  // In bits
    IN      unsigned char* Data,
    IN OUT  unsigned int* HashLength,  // In bits
    OUT     unsigned char* Hash
)
{
    NTSTATUS status = STATUS_SUCCESS;

    DEBUGOUTHASHIMPL(DEBUGLEV_ENTER, ("ImpHashHashData\n"));
    if (IsEqualGUID(&HASH_GUID_NULL, HashGUID))
        {
        if (*HashLength < DataLength)
            {
            DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("output hash length buffer too small (got: %d; need: %d)\n",
                *HashLength,
                DataLength
                ));
            status = STATUS_BUFFER_TOO_SMALL;
            }
        else
            {
            RtlZeroMemory(
                          Hash,
                          ((*HashLength) / 8)  // Convert bits to bytes
                         );

            RtlCopyMemory(            
                          Hash,
                          Data,
                          (DataLength / 8)  // Convert bits to bytes
                         );

            *HashLength = DataLength;
            }
            
        }
    else
        {
        DEBUGOUTHASHIMPL(DEBUGLEV_ERROR, ("driver doesn't recognise GUID\n"));
        status = STATUS_INVALID_PARAMETER;
        }
        
    DEBUGOUTHASHIMPL(DEBUGLEV_EXIT, ("ImpHashHashData\n"));
      
    return status;
}


// =========================================================================
// =========================================================================

