unit FreeOTFESettings;
// Description: 
// By Sarah Dean
// Email: sdean12@sdean12.org
// WWW:   http://www.FreeOTFE.org/
//
// -----------------------------------------------------------------------------
//


interface

uses
  INIFiles,
  Classes; // Required for TShortCut

type
  TFreeOTFESettings = class(TINIFile)
  public
    OptSaveSettings: boolean;

    // General...
    OptAllowMultipleInstances: boolean;
    OptDisplayToolbar: boolean;
    OptDisplayStatusbar: boolean;

    // System tray icon related...
    OptSystemTrayIconDisplay: boolean;
    OptSystemTrayIconMinTo: boolean;
    OptSystemTrayIconCloseTo: boolean;

    // Hotkey related...
    OptHKeyEnableDismount: boolean;
    OptHKeyKeyDismount: TShortCut;
    OptHKeyEnableDismountEmerg: boolean;
    OptHKeyKeyDismountEmerg: TShortCut;

    constructor Create();
    procedure Load();
    function Save(): boolean;

  end;

var
  // Global variable
  Settings: TFreeOTFESettings;

implementation

uses
  SysUtils,  // Required for ChangeFileExt
  Dialogs,
  SDUGeneral,
  Menus; // Required for ShortCutToText and TextToShortCut


const
  // -- General section --
  SECTION_GENERAL = 'General';
    OPT_SAVESETTINGS                             = 'SaveSettings';
      DFLT_OPT_SAVESETTINGS                         = FALSE;
    OPT_ALLOWMULTIPLEINSTANCES                   = 'AllowMultipleInstances';
      DFLT_OPT_ALLOWMULTIPLEINSTANCES               = FALSE;
    OPT_DISPLAYTOOLBAR                           = 'DisplayToolbar';
      DFLT_OPT_DISPLAYTOOLBAR                       = TRUE;
    OPT_DISPLAYSTATUSBAR                         = 'DisplayStatusbar';
      DFLT_OPT_DISPLAYSTATUSBAR                     = FALSE;

  // -- System tray icon section --
  SECTION_SYSTEMTRAYICON = 'SystemTrayIcon';
    OPT_SYSTEMTRAYICONDISPLAY                    = 'Display';
      DFLT_OPT_SYSTEMTRAYICONDISPLAY                = FALSE;
    OPT_SYSTEMTRAYICONMINTO                      = 'MinTo';
      DFLT_OPT_SYSTEMTRAYICONMINTO                  = FALSE;
    OPT_SYSTEMTRAYICONCLOSETO                    = 'CloseTo';
      DFLT_OPT_SYSTEMTRAYICONCLOSETO                = FALSE;

  // -- Hotkey section --
  SECTION_HOTKEY = 'Hotkeys';
    OPT_HOTKEYENABLEDISMOUNT                     = 'EnableDismount';
      DFLT_OPT_HOTKEYENABLEDISMOUNT                 = FALSE;
    OPT_HOTKEYKEYDISMOUNT                        = 'KeyDismount';
      DFLT_OPT_HOTKEYKEYDISMOUNT                    = 'Shift+Ctrl+D';
    OPT_HOTKEYENABLEDISMOUNTEMERG                = 'EnableDismountEmerg';
      DFLT_OPT_HOTKEYENABLEDISMOUNTEMERG            = FALSE;
    OPT_HOTKEYKEYDISMOUNTEMERG                   = 'KeyDismountEmerg';
      DFLT_OPT_HOTKEYKEYDISMOUNTEMERG               = 'Shift+Ctrl+Alt+D';



constructor TFreeOTFESettings.Create();
begin
  // Use the filename and path of the application, but change the filename
  // extension to ".ini"
  inherited Create(ChangeFileExt(ParamStr(0), '.ini'));

end;

procedure TFreeOTFESettings.Load();
begin
  OptSaveSettings           := ReadBool(SECTION_GENERAL, OPT_SAVESETTINGS,           DFLT_OPT_SAVESETTINGS);
  OptAllowMultipleInstances := ReadBool(SECTION_GENERAL, OPT_ALLOWMULTIPLEINSTANCES, DFLT_OPT_ALLOWMULTIPLEINSTANCES);
  OptDisplayToolbar         := ReadBool(SECTION_GENERAL, OPT_DISPLAYTOOLBAR,         DFLT_OPT_DISPLAYTOOLBAR);
  OptDisplayStatusbar       := ReadBool(SECTION_GENERAL, OPT_DISPLAYSTATUSBAR,       DFLT_OPT_DISPLAYSTATUSBAR);

  OptSystemTrayIconDisplay := ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONDISPLAY, DFLT_OPT_SYSTEMTRAYICONDISPLAY);
  OptSystemTrayIconMinTo   := ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONMINTO,   DFLT_OPT_SYSTEMTRAYICONMINTO);
  OptSystemTrayIconCloseTo := ReadBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONCLOSETO, DFLT_OPT_SYSTEMTRAYICONCLOSETO);

  OptHKeyEnableDismount      := ReadBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNT,      DFLT_OPT_HOTKEYENABLEDISMOUNT);
  OptHKeyKeyDismount         := TextToShortCut(ReadString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNT,         DFLT_OPT_HOTKEYKEYDISMOUNT));
  OptHKeyEnableDismountEmerg := ReadBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNTEMERG, DFLT_OPT_HOTKEYENABLEDISMOUNTEMERG);
  OptHKeyKeyDismountEmerg    := TextToShortCut(ReadString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNTEMERG,    DFLT_OPT_HOTKEYKEYDISMOUNTEMERG));

end;

function TFreeOTFESettings.Save(): boolean;
var
  allOK: boolean;
begin
  allOK := TRUE;
  if (OptSaveSettings) then
    begin
    try
      WriteBool(SECTION_GENERAL, OPT_SAVESETTINGS,           OptSaveSettings);
      WriteBool(SECTION_GENERAL, OPT_ALLOWMULTIPLEINSTANCES, OptAllowMultipleInstances);
      WriteBool(SECTION_GENERAL, OPT_DISPLAYTOOLBAR,         OptDisplayToolbar);
      WriteBool(SECTION_GENERAL, OPT_DISPLAYSTATUSBAR,       OptDisplayStatusbar);

      WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONDISPLAY, OptSystemTrayIconDisplay);
      WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONMINTO,   OptSystemTrayIconMinTo);
      WriteBool(SECTION_SYSTEMTRAYICON, OPT_SYSTEMTRAYICONCLOSETO, OptSystemTrayIconCloseTo);

      WriteBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNT,      OptHKeyEnableDismount);
      WriteString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNT,         ShortCutToText(OptHKeyKeyDismount));
      WriteBool(SECTION_HOTKEY,   OPT_HOTKEYENABLEDISMOUNTEMERG, OptHKeyEnableDismountEmerg);
      WriteString(SECTION_HOTKEY, OPT_HOTKEYKEYDISMOUNTEMERG,    ShortCutToText(OptHKeyKeyDismountEmerg));

    except
      on E: Exception do
        begin
        allOK := FALSE;
        MessageDlg(
                   'Your settings could not be saved.'+SDUCRLF+
                   SDUCRLF+
                   'Please check access rights to the ".ini" file stored with this application.',
                   mtError,
                   [mbOK],
                   0
                  );
        end;
    end;

    end;

  Result := allOK;
end;

END.


